@@ -19,7 +19,9 @@
  * Boston, MA 02111-1307, USA.
  */
 #include <config.h>
-#include <gnome.h>
+
+#include <glib/gi18n.h>
+
 #include <gnome-keyring.h>
 
 #include "seahorse-context.h"
@@ -155,7 +157,7 @@ update_default_keyring (SeahorseWidget *swidget)
         gtk_tree_model_get (model, &iter, KEYRING_NAME, &t, -1);
         g_assert (t);
         
-        match = (strcmp (t, keyring) == 0);
+        match = (g_strcmp0 (t, keyring) == 0);
         g_free (t);
          
         if (match) {
@@ -194,14 +196,14 @@ refresh_keyring_model (SeahorseWidget *swidget, GList *list)
     
         /* We don't list the 'session' keyring */
         name = (gchar*)list->data;
-        if (strcmp (name, "session") == 0)
+        if (g_strcmp0 (name, "session") == 0)
             continue;
 
         if (!valid)
             gtk_list_store_append (store, &iter);
 
         /* Figure out the name and markup */
-        if (strcmp (name, "login") == 0)
+        if (g_strcmp0 (name, "login") == 0)
             comments = _("<i>Automatically unlocked when user logs in.</i>");
         else
             comments = "";
@@ -238,7 +240,7 @@ refreshed_default (GnomeKeyringResult result, const gchar* name, gpointer data)
         update_default_keyring (swidget);
         
     } else if (result != GNOME_KEYRING_RESULT_CANCELLED) {
-        seahorse_util_show_error (GTK_WINDOW (seahorse_widget_get_top (swidget)),
+        seahorse_util_show_error (seahorse_widget_get_toplevel (swidget),
                                   _("Couldn't get default password keyring"),
                                   gnome_keyring_result_to_message (result));
     }    
@@ -261,7 +263,7 @@ refreshed_keyrings (GnomeKeyringResult result, GList *list, gpointer data)
         update_default_keyring (swidget);
         
     } else if (result != GNOME_KEYRING_RESULT_CANCELLED) {
-        seahorse_util_show_error (GTK_WINDOW (seahorse_widget_get_top (swidget)),
+        seahorse_util_show_error (seahorse_widget_get_toplevel (swidget),
                                   _("Couldn't get list of password keyrings"),
                                   gnome_keyring_result_to_message (result));
     }
@@ -293,7 +295,7 @@ static void
 keyring_default_done (GnomeKeyringResult result, gpointer data)
 {
     SeahorseWidget *swidget;
-    SeahorseKeySource *src;
+    SeahorseSource *src;
 
     swidget = SEAHORSE_WIDGET (data);
     g_return_if_fail (swidget);
@@ -307,17 +309,16 @@ keyring_default_done (GnomeKeyringResult result, gpointer data)
         
         /* 
          * Update the main seahorse gkeyring source 
-         * TODO: We shouldn't have to define SKEY_GKEYRING here again
+         * TODO: We shouldn't have to define SEAHORSE_GKR here again
          */
-        src = seahorse_context_find_key_source (SCTX_APP (), 
-                                                g_quark_from_static_string ("gnome-keyring"), 
-                                                SKEY_LOC_LOCAL);
+        src = seahorse_context_find_source (SCTX_APP (), g_quark_from_static_string ("gnome-keyring"), 
+                                            SEAHORSE_LOCATION_LOCAL);
         if (src)
-            seahorse_key_source_load_async (src, 0);
+            seahorse_source_load_async (src, 0);
         
     /* Setting the default keyring failed */
     } else if (result != GNOME_KEYRING_RESULT_CANCELLED) {     
-        seahorse_util_show_error (GTK_WINDOW (seahorse_widget_get_top (swidget)),
+        seahorse_util_show_error (seahorse_widget_get_toplevel (swidget),
                                   _("Couldn't set default password keyring"),
                                   gnome_keyring_result_to_message (result));
     }
@@ -399,7 +400,7 @@ keyring_remove_done (GnomeKeyringResult result, gpointer data)
         
     /* Setting the default keyring failed */
     } else if (result != GNOME_KEYRING_RESULT_CANCELLED) {     
-        seahorse_util_show_error (GTK_WINDOW (seahorse_widget_get_top (swidget)),
+        seahorse_util_show_error (seahorse_widget_get_toplevel (swidget),
                                   _("Couldn't remove keyring"),
                                   gnome_keyring_result_to_message (result));
     }
@@ -416,7 +417,7 @@ keyring_remove_clicked (GtkButton *button, SeahorseWidget *swidget)
     keyring = selected_keyring (swidget);
     g_return_if_fail (keyring);
 
-    question = gtk_message_dialog_new (GTK_WINDOW (seahorse_widget_get_top (swidget)), 
+    question = gtk_message_dialog_new (GTK_WINDOW (seahorse_widget_get_toplevel (swidget)), 
                                        GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE,
                                        _("Are you sure you want to permanently delete the '%s' keyring?"), 
                                        keyring);
@@ -465,7 +466,7 @@ keyring_add_done (GnomeKeyringResult result, gpointer data)
         
     /* Setting the default keyring failed */
     } else if (result != GNOME_KEYRING_RESULT_CANCELLED) {     
-        seahorse_util_show_error (GTK_WINDOW (seahorse_widget_get_top (swidget)),
+        seahorse_util_show_error (seahorse_widget_get_toplevel (swidget),
                                   _("Couldn't add keyring"),
                                   gnome_keyring_result_to_message (result));
     }
@@ -481,7 +482,7 @@ keyring_add_clicked (GtkButton *button, SeahorseWidget *swidget)
     gint response;
     
     swadd = seahorse_widget_new_allow_multiple ("add-keyring",
-                                                GTK_WINDOW (seahorse_widget_get_top (swidget)));
+                                                GTK_WINDOW (seahorse_widget_get_toplevel (swidget)));
     g_return_if_fail (swadd);
     
     entry = GTK_ENTRY (seahorse_widget_get_widget (swadd, "keyring-name"));
@@ -491,7 +492,7 @@ keyring_add_clicked (GtkButton *button, SeahorseWidget *swidget)
                                    G_CALLBACK (keyring_name_changed), swadd);
     keyring_name_changed (entry, swadd);
 
-    response = gtk_dialog_run (GTK_DIALOG (seahorse_widget_get_top (swadd)));
+    response = gtk_dialog_run (GTK_DIALOG (seahorse_widget_get_toplevel (swadd)));
     if (response == GTK_RESPONSE_ACCEPT) {
         
         keyring = gtk_entry_get_text (entry);
@@ -522,7 +523,7 @@ keyring_password_done (GnomeKeyringResult result, gpointer data)
         
     /* Setting the default keyring failed */
     } else if (result != GNOME_KEYRING_RESULT_CANCELLED) {     
-        seahorse_util_show_error (GTK_WINDOW (seahorse_widget_get_top (swidget)),
+        seahorse_util_show_error (seahorse_widget_get_toplevel (swidget),
                                   _("Couldn't change keyring password"),
                                   gnome_keyring_result_to_message (result));
     }
