@@ -2,7 +2,7 @@
  * Seahorse
  *
  * Copyright (C) 2002 Jacob Perkins
- * Copyright (C) 2005 Nate Nielsen
+ * Copyright (C) 2005 Stefan Walter
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,11 +21,16 @@
  */
 
 #include <config.h>
-#include <gnome.h>
+
+#include <string.h>
+
+#include <glib/gi18n.h>
+
 #include <glade/glade.h>
 #include <glade/glade-build.h>
 
 #include "seahorse-widget.h"
+#include "seahorse-gconf.h"
 #include "seahorse-gtkstock.h"
 
 #define STATUS "status"
@@ -49,6 +54,10 @@ static void     object_get_property (GObject                *object,
                                      guint                  prop_id,
                                      GValue                 *value,
                                      GParamSpec             *pspec);
+                                     
+static GObject* seahorse_widget_constructor (GType                  type, 
+                                             guint                  n_props, 
+                                             GObjectConstructParam* props);                    
 
 /* signal functions */
 static void     widget_closed        (GtkWidget             *widget,
@@ -96,6 +105,7 @@ class_init (SeahorseWidgetClass *klass)
 	parent_class = g_type_class_peek_parent (klass);
 	gobject_class = G_OBJECT_CLASS (klass);
 	
+	gobject_class->constructor = seahorse_widget_constructor;
 	gobject_class->finalize = object_finalize;
 	gobject_class->set_property = object_set_property;
 	gobject_class->get_property = object_get_property;
@@ -119,6 +129,36 @@ object_init (SeahorseWidget *swidget)
                 G_CALLBACK (context_destroyed), swidget);
 }
 
+static GObject*  
+seahorse_widget_constructor (GType type, guint n_props, GObjectConstructParam* props)
+{
+    SeahorseWidget *swidget;
+    GObject *obj;
+    
+    GtkWindow *window;
+    gint width, height;
+    gchar *widthkey, *heightkey;
+    
+    obj = G_OBJECT_CLASS (parent_class)->constructor (type, n_props, props);
+    swidget = SEAHORSE_WIDGET (obj);
+
+    widthkey = g_strdup_printf ("%s%s%s", WINDOW_SIZE, swidget->name, "_width");
+    width = seahorse_gconf_get_integer (widthkey);
+    
+    heightkey = g_strdup_printf ("%s%s%s", WINDOW_SIZE, swidget->name, "_height");
+    height = seahorse_gconf_get_integer (heightkey);
+
+    if (width != 0 && height != 0) {
+        window = GTK_WINDOW (seahorse_widget_get_toplevel (swidget));
+        gtk_window_resize (window, width, height);
+    }
+    
+    g_free (widthkey);
+    g_free (heightkey);
+    
+    return obj;
+}
+
 /* Disconnects callbacks, destroys main window widget,
  * and frees the xml definition and any other data */
 static void
@@ -131,7 +171,7 @@ object_finalize (GObject *gobject)
 	/* Remove widget from hash and destroy hash if empty */
     if (widgets) {
     	g_hash_table_remove (widgets, swidget->name);
-    	if (g_hash_table_size == 0) {
+    	if (g_hash_table_size (widgets) == 0) {
     		g_hash_table_destroy (widgets);
     		widgets = NULL;
     	}
@@ -228,16 +268,18 @@ widget_delete_event (GtkWidget *widget, GdkEvent *event, SeahorseWidget *swidget
 /**
  * seahorse_widget_new:
  * @name: Name of widget, filename part of glade file, and name of main window
+ * @parent: GtkWindow to make the parent of the new swidget
  *
  * Creates a new #SeahorseWidget.
  *
  * Returns: The new #SeahorseWidget, or NULL if the widget already exists
  **/
 SeahorseWidget*
-seahorse_widget_new (const gchar *name)
+seahorse_widget_new (const gchar *name, GtkWindow *parent)
 {
-    /* Check if have widget hash */
+        /* Check if have widget hash */
     SeahorseWidget *swidget = seahorse_widget_find (name);
+    GtkWindow *window;
     
     /* If widget already exists, present */
     if (swidget != NULL) {
@@ -250,6 +292,11 @@ seahorse_widget_new (const gchar *name)
     if(!widgets)
         widgets = g_hash_table_new ((GHashFunc)g_str_hash, (GCompareFunc)g_str_equal);
     g_hash_table_insert (widgets, g_strdup (name), swidget);
+    
+    if (parent != NULL) {
+        window = GTK_WINDOW (glade_xml_get_widget (swidget->xml, swidget->name));
+        gtk_window_set_transient_for (window, parent);
+    }
 
     /* We don't care about this floating business */
     g_object_ref (GTK_OBJECT (swidget));
@@ -259,18 +306,25 @@ seahorse_widget_new (const gchar *name)
 }
 
 /**
- * seahorse_widget_new:
+ * seahorse_widget_new_allow_multiple:
  * @name: Name of widget, filename part of glade file, and name of main window
+ * @parent: GtkWindow to make the parent of the new swidget
  *
  * Creates a new #SeahorseWidget without checking if it already exists.
  *
  * Returns: The new #SeahorseWidget
  **/
 SeahorseWidget*
-seahorse_widget_new_allow_multiple (const gchar *name)
+seahorse_widget_new_allow_multiple (const gchar *name, GtkWindow *parent)
 {
+    GtkWindow *window;
     SeahorseWidget *swidget = g_object_new (SEAHORSE_TYPE_WIDGET, "name", name,  NULL);
     
+    if (parent != NULL) {
+        window = GTK_WINDOW (glade_xml_get_widget (swidget->xml, swidget->name));
+        gtk_window_set_transient_for (window, parent);
+    }
+    
     /* We don't care about this floating business */
     g_object_ref (GTK_OBJECT (swidget));
     gtk_object_sink (GTK_OBJECT (swidget));
@@ -296,30 +350,40 @@ seahorse_widget_find (const gchar *name)
 void
 seahorse_widget_show_help (SeahorseWidget *swidget)
 {
-    GError *err = NULL;
+    GError *error = NULL;
+    gchar *document = NULL;
+    GtkWidget *dialog = NULL;
 
     if (g_str_equal (swidget->name, "key-manager") || 
-        g_str_equal (swidget->name, "keyserver-results"))
-        gnome_help_display (PACKAGE, "introduction", &err);
-    else
-        gnome_help_display (PACKAGE, swidget->name, &err);
-
-    if (err != NULL) {
-        GtkWidget *dialog;
-
-        dialog = gtk_message_dialog_new (GTK_WINDOW (seahorse_widget_get_top (swidget)), GTK_DIALOG_MODAL, 
-                                         GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, 
-                                         _("Could not display help: %s"),
-                                         err->message);
-        g_signal_connect (G_OBJECT (dialog), "response",
-                          G_CALLBACK (gtk_widget_destroy), NULL);
-        gtk_widget_show (dialog);
-        g_error_free (err);
+        g_str_equal (swidget->name, "keyserver-results")) {
+        document = g_strdup ("ghelp:" PACKAGE "?introduction");
+    } else {
+        document = g_strdup_printf ("ghelp:" PACKAGE "?%s", swidget->name);
     }
+
+    if (!g_app_info_launch_default_for_uri (document, NULL, &error)) {
+        dialog = gtk_message_dialog_new (GTK_WINDOW (seahorse_widget_get_toplevel (swidget)),
+                                         GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
+                                         _("Could not display help: %s"), error->message);
+        g_signal_connect (G_OBJECT (dialog), "response", G_CALLBACK (gtk_widget_destroy), NULL);
+        gtk_widget_show (dialog);
+    }
+
+    g_free (document);
+
+    if (error)
+        g_error_free (error);
+}
+
+const gchar*
+seahorse_widget_get_name (SeahorseWidget   *swidget)
+{
+	g_return_val_if_fail (SEAHORSE_IS_WIDGET (swidget), NULL);
+	return swidget->name;
 }
 
 /**
- * seahorse_widget_get_top
+ * seahorse_widget_get_toplevel
  * @swidget: The seahorse widget
  * 
  * Return the top level widget in this seahorse widget
@@ -327,7 +391,7 @@ seahorse_widget_show_help (SeahorseWidget *swidget)
  * Returns: The top level widget
  **/
 GtkWidget*      
-seahorse_widget_get_top     (SeahorseWidget     *swidget)
+seahorse_widget_get_toplevel (SeahorseWidget     *swidget)
 {
     GtkWidget *widget = glade_xml_get_widget (swidget->xml, swidget->name);
     g_return_val_if_fail (widget != NULL, NULL);
@@ -353,9 +417,6 @@ seahorse_widget_show (SeahorseWidget *swidget)
 {
     GtkWidget *widget;
 
-    if (swidget->ui)
-        gtk_ui_manager_ensure_update (swidget->ui);
-
     widget = glade_xml_get_widget (swidget->xml, swidget->name);
     g_return_if_fail (widget != NULL);
     gtk_widget_show (widget);
@@ -374,6 +435,15 @@ seahorse_widget_set_visible (SeahorseWidget *swidget, const char *identifier,
         gtk_widget_hide (widget);
 }
 
+void             
+seahorse_widget_set_sensitive (SeahorseWidget *swidget, const char *identifier,
+                               gboolean sensitive)
+{
+    GtkWidget *widget = glade_xml_get_widget (swidget->xml, identifier);
+    g_return_if_fail (widget != NULL);
+    gtk_widget_set_sensitive (widget, sensitive);
+}
+
 /**
  * seahorse_widget_destroy:
  * @swidget: #SeahorseWidget to destroy
@@ -383,163 +453,36 @@ seahorse_widget_set_visible (SeahorseWidget *swidget, const char *identifier,
 void
 seahorse_widget_destroy (SeahorseWidget *swidget)
 {
+    GtkWidget *widget;
+    gchar *widthkey, *heightkey;
+    gint width, height;
+
     g_return_if_fail (swidget != NULL && SEAHORSE_IS_WIDGET (swidget));
+    
+    /* Save window size */
+    widget = seahorse_widget_get_toplevel (swidget);
+    gtk_window_get_size (GTK_WINDOW (widget), &width, &height);
+    
+    widthkey = g_strdup_printf ("%s%s%s", WINDOW_SIZE, swidget->name, "_width");
+    seahorse_gconf_set_integer (widthkey, width);
+    
+    heightkey = g_strdup_printf ("%s%s%s", WINDOW_SIZE, swidget->name, "_height");
+    seahorse_gconf_set_integer (heightkey, height);
+    
+    g_free (widthkey);
+    g_free (heightkey);
+    
+    /* Destroy Widget */
     if (!swidget->destroying) {
         swidget->destroying = TRUE;
         g_object_unref (swidget);
     }
 }
 
-/* UI MANAGER CODE ---------------------------------------------------------- */
-
-static void
-ui_add_widget (GtkUIManager *ui, GtkWidget *widget, SeahorseWidget *swidget)
+void
+seahorse_widget_connect_glade_signal (SeahorseWidget *swidget, const char *event, 
+                                      GtkCallback callback, gpointer userdata)
 {
-    GtkWidget *holder;
-    const gchar *name;
-    
-    /* We automatically add menus and toolbars */
-    if (GTK_IS_MENU_BAR (widget))
-        name = "menu-placeholder";
-    else
-        name = "toolbar-placeholder";
-    
-    if (name != NULL) {
-        /* Find the appropriate position in the glade file */
-        holder = glade_xml_get_widget (swidget->xml, name);
-        if (holder != NULL)
-            gtk_container_add (GTK_CONTAINER (holder), widget);
-        else
-            g_warning ("no place holder found for: %s", name);
-    }
-}
-
-static void
-ui_load (SeahorseWidget *swidget)
-{
-    GtkWidget *w;
-    GError *err = NULL;
-    gchar *path;
-    
-    if (!swidget->ui) {
-        
-        /* Load the menu/toolbar description file */
-        swidget->ui = gtk_ui_manager_new ();
-    	path = g_strdup_printf ("%sseahorse-%s.ui", SEAHORSE_GLADEDIR, swidget->name);
-        gtk_ui_manager_add_ui_from_file (swidget->ui, path, &err);
-		g_free (path);
-        
-        if (err) {
-            g_warning ("couldn't load ui description for '%s': %s", swidget->name, err->message);
-            g_error_free (err);
-            return;
-        }
-
-        /* The widgets get added in an idle loop later */
-        g_signal_connect (swidget->ui, "add-widget", G_CALLBACK (ui_add_widget), swidget);
-        
-        /* Attach accelerators to the window */
-        w = glade_xml_get_widget (swidget->xml, swidget->name);
-        if (GTK_IS_WINDOW (w))
-            gtk_window_add_accel_group (GTK_WINDOW (w), gtk_ui_manager_get_accel_group (swidget->ui));
-    }    
-}
-
-static void
-cleanup_actions (GtkActionGroup *group)
-{
-    GList *actions, *l;
-    
-    #define ELIPSIS "..."
-    #define ELIPSIS_LEN 3
-    
-    actions = gtk_action_group_list_actions (group);    
-    
-    for (l = actions; l; l = g_list_next (l)) {
-        GtkAction *action = GTK_ACTION (l->data);
-        gchar *label;
-        guint len;
-        
-        /* Remove the ellipsis from the end of action labels if present */
-        g_object_get (action, "short-label", &label, NULL);
-        if (label) {
-            len = strlen (label);
-            if (strcmp (ELIPSIS, label + (len - ELIPSIS_LEN)) == 0) {
-                label[len - ELIPSIS_LEN] = 0;
-                g_object_set (action, "short-label", label, NULL);
-            }
-            g_free (label);
-        }
-    }
- 
-    g_list_free (actions);    
-}
-
-/**
- * seahorse_widget_get_ui_widget
- * @swidget: The #SeahorseWidget.
- * @path: The path to the widget. See gtk_ui_manager_get_widget
- * 
- * Returns a piece of generated UI. Note this doesn't look in the glade
- * file but rather looks in the GtkUIManager UI. If no UI has been loaded 
- * then one will be loaded. The UI file has the same name as the glade file 
- * but with a 'ui' extension. 
- */
-GtkWidget*
-seahorse_widget_get_ui_widget (SeahorseWidget *swidget, const gchar *path)
-{
-    g_return_val_if_fail (SEAHORSE_IS_WIDGET (swidget), NULL);
-    
-    ui_load (swidget);    
-    g_return_val_if_fail (swidget->ui, NULL);
-    
-    return gtk_ui_manager_get_widget (swidget->ui, path);
-}
-
-/**
- * seahorse_widget_add_actions
- * @swidget: The #SeahorseWidget.
- * @actions: A #GtkActionGroup to add to the UI.
- * 
- * Adds a GtkActionGroup to this widget's GtkUIManager UI. If no UI
- * has been loaded then one will be loaded. The UI file has the same
- * name as the glade file but with a 'ui' extension. 
- */
-void             
-seahorse_widget_add_actions (SeahorseWidget *swidget, GtkActionGroup *actions)
-{
-    g_return_if_fail (SEAHORSE_IS_WIDGET (swidget));
-    
-    ui_load (swidget);    
-    g_return_if_fail (swidget->ui);
-
-    cleanup_actions (actions);
-    gtk_ui_manager_insert_action_group (swidget->ui, actions, -1);
-}
-
-/** 
- * seahorse_widget_find_actions
- * @swidget: The #SeahorseWidget.
- * @name: The name of the action group.
- * 
- * Find an #GtkActionGroup previously added to this widget.
- * 
- * Returns: The action group.
- */
-GtkActionGroup*
-seahorse_widget_find_actions (SeahorseWidget *swidget, const gchar *name)
-{
-    GList *l;
-    
-    g_return_val_if_fail (SEAHORSE_IS_WIDGET (swidget), NULL);
-    
-    if (!swidget->ui)
-        return NULL;
-    
-    for (l = gtk_ui_manager_get_action_groups (swidget->ui); l; l = g_list_next (l)) {
-        if (g_str_equal (gtk_action_group_get_name (GTK_ACTION_GROUP (l->data)), name)) 
-            return GTK_ACTION_GROUP (l->data);
-    }
-    
-    return NULL;
+	g_return_if_fail (SEAHORSE_IS_WIDGET (swidget));
+	glade_xml_signal_connect_data (swidget->xml, event, G_CALLBACK (callback), userdata);
 }
