@@ -21,37 +21,33 @@
  */
 
 #include "config.h"
-#include <gnome.h>
 
+#include <string.h>
+#include <unistd.h>
+
+#include <glib/gi18n.h>
+
+#include "seahorse-gconf.h"
 #include "seahorse-key-manager-store.h"
 #include "seahorse-preferences.h"
-#include "seahorse-validity.h"
 #include "seahorse-util.h"
-#include "seahorse-gconf.h"
-#include "seahorse-gpgmex.h"
-#include "eggtreemultidnd.h"
-#include "seahorse-pgp-key.h"
-#include "seahorse-vfs-data.h"
+#include "seahorse-validity.h"
 
-#ifdef WITH_SSH
-#include "seahorse-ssh-key.h"
-#endif 
+#include "eggtreemultidnd.h"
 
 #define KEY_MANAGER_SORT_KEY "/apps/seahorse/listing/sort_by"
 
 enum {
     PROP_0,
-    PROP_KEYSET,
     PROP_MODE,
     PROP_FILTER
 };
 
 enum {
-    COL_PAIR,
+    COL_USAGE,
     COL_STOCK_ID,
     COL_NAME,
     COL_KEYID,
-    COL_UID,
     COL_VALIDITY_STR,
     COL_TRUST_STR,
     COL_TYPE,
@@ -62,34 +58,33 @@ enum {
     N_COLS
 };
 
-static const gchar* col_ids[] = {
-    "pair",
-    NULL,
-    "name",
-    "id",
-    NULL,
-    "validity",
-    "trust",
-    "type",
-    "expires",
-    "validity",
-    "expires",
-    "trust"
+static const SeahorseSetModelColumn column_info[] = {
+	{ "usage", G_TYPE_INT, "usage" },
+	{ "stock-id", G_TYPE_STRING, NULL },
+	{ "markup", G_TYPE_STRING, "name" },
+	{ "display-id", G_TYPE_STRING, "id" },
+	{ "validity-str", G_TYPE_STRING, "validity" },
+	{ "trust-str", G_TYPE_STRING, "trust" },
+	{ "type", G_TYPE_STRING, "type" },
+	{ "expires-str", G_TYPE_STRING, "expires" },
+	{ "validity", G_TYPE_INT, "validity" },
+	{ "expires", G_TYPE_LONG, "expires" },
+	{ "trust", G_TYPE_INT, "trust" }
 };
 
-static const GType col_types[] = {
-    G_TYPE_BOOLEAN,
-    G_TYPE_STRING,
-    G_TYPE_STRING,
-    G_TYPE_STRING,
-    G_TYPE_INT,
-    G_TYPE_STRING,
-    G_TYPE_STRING, 
-    G_TYPE_STRING, 
-    G_TYPE_STRING,
-    G_TYPE_INT, 
-    G_TYPE_LONG, 
-    G_TYPE_INT
+enum {
+	DRAG_INFO_TEXT,
+	DRAG_INFO_XDS,
+};
+
+#define XDS_FILENAME "xds.txt"
+#define MAX_XDS_ATOM_VAL_LEN 4096
+#define XDS_ATOM   gdk_atom_intern  ("XdndDirectSave0", FALSE)
+#define TEXT_ATOM  gdk_atom_intern  ("text/plain", FALSE)
+
+static GtkTargetEntry store_targets[] = {
+	{ "text/plain", 0, DRAG_INFO_TEXT },
+	{ "XdndDirectSave0", 0, DRAG_INFO_XDS }
 };
 
 struct _SeahorseKeyManagerStorePriv {    
@@ -99,9 +94,13 @@ struct _SeahorseKeyManagerStorePriv {
     SeahorseKeyManagerStoreMode    filter_mode;
     gchar*                  filter_text;
     guint                   filter_stag;
+    
+	gchar *drag_destination;
+	GError *drag_error;
+	GList *drag_objects;
 };
 
-G_DEFINE_TYPE (SeahorseKeyManagerStore, seahorse_key_manager_store, SEAHORSE_TYPE_KEY_MODEL);
+G_DEFINE_TYPE (SeahorseKeyManagerStore, seahorse_key_manager_store, SEAHORSE_TYPE_SET_MODEL);
 
 /* -----------------------------------------------------------------------------
  * INTERNAL 
@@ -163,26 +162,22 @@ key_store_from_model (GtkTreeModel *model)
     return NULL;
 }
 
-/* Given an iterator find the associated key */
-static SeahorseKey*
-key_from_iterator (GtkTreeModel* model, GtkTreeIter* iter, guint *uid)
+/* Given an iterator find the associated object */
+static SeahorseObject*
+object_from_iterator (GtkTreeModel* model, GtkTreeIter* iter)
 {
-    GtkTreeIter i;
-    SeahorseKey *skey = NULL;
+	GtkTreeIter i;
     
-    /* Convert to base iter if necessary */
-    if (!SEAHORSE_IS_KEY_MANAGER_STORE (model)) {
-        SeahorseKeyManagerStore* skstore = key_store_from_model (model);
-        get_base_iter (skstore, &i, iter);
+	/* Convert to base iter if necessary */
+	if (!SEAHORSE_IS_KEY_MANAGER_STORE (model)) {
+		SeahorseKeyManagerStore* skstore = key_store_from_model (model);
+		get_base_iter (skstore, &i, iter);
         
-        iter = &i;
-        model = GTK_TREE_MODEL (skstore);
-    }
+		iter = &i;
+		model = GTK_TREE_MODEL (skstore);
+	}
     
-    skey = seahorse_key_model_get_row_key (SEAHORSE_KEY_MODEL (model), iter);
-    if (skey && uid)
-        gtk_tree_model_get (model, iter, COL_UID, uid, -1);
-    return skey;
+	return seahorse_set_model_object_for_iter (SEAHORSE_SET_MODEL (model), iter);
 }
 
 /* Search through row for text */
@@ -258,7 +253,7 @@ filter_callback (GtkTreeModel *model, GtkTreeIter *iter, gpointer data)
 static gboolean
 refilter_now (SeahorseKeyManagerStore* skstore)
 {
-    seahorse_keyset_refresh (skstore->skset);
+    seahorse_set_refresh (SEAHORSE_SET_MODEL (skstore)->set);
     gtk_tree_model_filter_refilter (skstore->priv->filter);    
     skstore->priv->filter_stag = 0;
     return FALSE;
@@ -273,186 +268,6 @@ refilter_later (SeahorseKeyManagerStore* skstore)
     skstore->priv->filter_stag = g_timeout_add (200, (GSourceFunc)refilter_now, skstore);
 }
 
-/* Sets Name and KeyID */
-static void
-update_key_row (SeahorseKeyManagerStore *skstore, SeahorseKey *skey, guint uid, 
-                GtkTreeIter *iter)
-{
-    SeahorseValidity validity, trust;
-    SeahorseKeyPredicate *pred;
-    SeahorseKeySource *sksrc;
-    const gchar *stockid;
-    gulong expires_date;
-    gchar *markup;
-    gboolean sec;
-    gchar *expires;
-    gchar *type;
-
-    markup = seahorse_key_get_name_markup (skey, uid);
-    sec = seahorse_key_get_etype (skey) == SKEY_PRIVATE;
-    stockid = seahorse_key_get_stock_id (skey);
-    validity = seahorse_key_get_name_validity (skey, uid);
-    
-    if (uid == 0) {
-        
-        trust = seahorse_key_get_trust (skey);
-        
-        if (seahorse_key_get_flags (skey) & SKEY_FLAG_EXPIRED) {
-            expires = g_strdup (_("Expired"));
-            expires_date = -1;
-        } else {
-            expires_date = seahorse_key_get_expires (skey);
-            
-            if (expires_date == 0) {
-                expires = g_strdup ("");
-                expires_date = G_MAXLONG;
-            }
-            else
-                expires = seahorse_util_get_display_date_string (expires_date);
-        }
-        
-        /* Only differentiate if the view shows more than one type of key */
-        g_object_get (skstore->skset, "predicate", &pred, NULL);
-        
-        /* If mixed etypes, then get specific description */
-        if (pred->etype == 0 || pred->etype == SKEY_CREDENTIALS) {
-            type = g_strdup (seahorse_key_get_desc (skey));
-            
-        /* Otherwise general description */
-        } else {
-            sksrc = seahorse_key_get_source (skey);
-            g_return_if_fail (sksrc);
-            g_object_get (sksrc, "key-desc", &type, NULL);
-        }
-
-        gtk_tree_store_set (GTK_TREE_STORE (skstore), iter,
-            COL_PAIR, uid == 0 ? sec : FALSE,
-            COL_STOCK_ID, uid == 0 ? stockid : NULL,
-            COL_NAME, markup,
-            COL_KEYID, seahorse_key_get_short_keyid (skey),
-            COL_UID, uid,
-            COL_VALIDITY_STR, validity == SEAHORSE_VALIDITY_UNKNOWN ? "" : seahorse_validity_get_string (validity),
-            COL_VALIDITY, validity,
-            COL_TRUST_STR, trust == SEAHORSE_VALIDITY_UNKNOWN ? "" : seahorse_validity_get_string (trust),
-            COL_TRUST, trust,
-            COL_TYPE, type,
-            COL_EXPIRES_STR, expires ? expires : "",
-            COL_EXPIRES, expires_date,
-            -1);
-         
-        g_free (type);
-        g_free (expires);
-       
-    } else {
-
-        gtk_tree_store_set (GTK_TREE_STORE (skstore), iter,
-            COL_KEYID, "", 
-            COL_NAME, markup,
-            COL_UID, uid,
-            COL_VALIDITY_STR, validity == SEAHORSE_VALIDITY_UNKNOWN ? "" : seahorse_validity_get_string (validity),
-            COL_VALIDITY, validity,
-            COL_TYPE, _("User ID"), 
-            -1);
-    }
-    
-    g_free (markup);
-}
-
-static gboolean
-append_key_row (SeahorseKeyManagerStore *skstore, SeahorseKey *skey, guint uid, 
-                GtkTreeIter *iter)
-{
-    GtkTreeIter child;
-
-    if (uid == 0) {
-        gtk_tree_store_append (GTK_TREE_STORE (skstore), iter, NULL);
-        update_key_row (skstore, skey, uid, iter);
-        seahorse_key_model_set_row_key (SEAHORSE_KEY_MODEL (skstore), iter, skey);
-    } else {
-        gtk_tree_store_append (GTK_TREE_STORE (skstore), &child, iter);
-        update_key_row (skstore, skey, uid, &child);
-        seahorse_key_model_set_row_key (SEAHORSE_KEY_MODEL (skstore), &child, skey);
-    }
-    
-    return TRUE;
-}
-
-static void
-key_added (SeahorseKeyset *skset, SeahorseKey *skey, SeahorseKeyManagerStore *skstore)
-{
-    GtkTreeIter iter = { 0, };
-    guint i, uids = seahorse_key_get_num_names (skey);
- 
-    for (i = 0; i < uids; i++) {
-        if (!append_key_row (skstore, skey, i, &iter))
-            break;
-    }
-}
-
-static void
-key_removed (SeahorseKeyset *skset, SeahorseKey *skey, gpointer closure, 
-             SeahorseKeyManagerStore *skstore)
-{
-    seahorse_key_model_remove_rows_for_key (SEAHORSE_KEY_MODEL (skstore), skey);
-}
-
-static void
-key_changed (SeahorseKeyset *skset, SeahorseKey *skey, SeahorseKeyChange change, 
-             gpointer closure, SeahorseKeyManagerStore *skstore)
-{
-    guint i, uid, old_uids, num_uids;
-    GtkTreeIter first;
-    GtkTreeIter *iter;
-    GSList *rows, *l;
-    
-    old_uids = 0;
-    num_uids = seahorse_key_get_num_names (skey);
-    rows = seahorse_key_model_get_rows_for_key (SEAHORSE_KEY_MODEL (skstore), skey);
-    
-    for (l = rows; l; l = g_slist_next (l)) {
-        
-        iter = (GtkTreeIter*)l->data;
-        gtk_tree_model_get (GTK_TREE_MODEL (skstore), iter, COL_UID, &uid, -1);
-        
-        /* Remove any extra rows */
-        if (uid >= num_uids) {
-            seahorse_key_model_set_row_key (SEAHORSE_KEY_MODEL (skstore), iter, NULL);
-            gtk_tree_store_remove (GTK_TREE_STORE (skstore), iter);
-            continue;
-        }
-
-        update_key_row (skstore, skey, uid, iter);
-        
-        /* The top parent row */
-        if (uid == 0)
-            memcpy (&first, iter, sizeof (first));            
-
-        /* Find the max uid on the rows */
-        if (uid >= old_uids)
-            old_uids = uid + 1;
-    }
-
-    /* Add all new rows */    
-    for (i = old_uids; i < num_uids; i++)
-        append_key_row (skstore, skey, i, &first);
-    
-    seahorse_key_model_free_rows (rows);
-}
-
-static void
-populate_store (SeahorseKeyManagerStore *skstore)
-{
-    GList *keys, *list = NULL;
-    SeahorseKey *skey;
-    
-    keys = list = seahorse_keyset_get_keys (skstore->skset);
-    while (list != NULL && (skey = list->data) != NULL) {
-        key_added (skstore->skset, skey, skstore);
-        list = g_list_next (list);
-    }
-    g_list_free (keys);
-}
-
 /* Update the sort order for a column */
 static void
 set_sort_to (SeahorseKeyManagerStore *skstore, const gchar *name)
@@ -476,9 +291,9 @@ set_sort_to (SeahorseKeyManagerStore *skstore, const gchar *name)
         name++;
     }
     
-    /* Find the column id */
+    /* Find the column sort id */
     for (i = N_COLS - 1; i >= 0 ; i--) {
-        n = col_ids[i];
+        n = column_info[i].data;
         if (n && g_ascii_strcasecmp (name, n) == 0) {
             id = i;
             break;
@@ -506,7 +321,7 @@ sort_changed (GtkTreeSortable *sort, gpointer user_data)
     /* We have a sort so save it */
     if (gtk_tree_sortable_get_sort_column_id (sort, &id, &ord)) {
         if (id >= 0 && id < N_COLS) {
-            t = col_ids[id];
+            t = column_info[id].data;
             if (t != NULL) {
                 x = g_strconcat (ord == GTK_SORT_DESCENDING ? "-" : "", t, NULL);
                 seahorse_gconf_set_string (KEY_MANAGER_SORT_KEY, x);
@@ -561,7 +376,7 @@ gconf_notification (GConfClient *gclient, guint id, GConfEntry *entry, GtkTreeVi
 
 static GtkTreeViewColumn*
 append_text_column (SeahorseKeyManagerStore *skstore, GtkTreeView *view, 
-               const gchar *label, const gint index)
+                    const gchar *label, const gint index)
 {
     GtkTreeViewColumn *column;
     GtkCellRenderer *renderer;
@@ -575,161 +390,268 @@ append_text_column (SeahorseKeyManagerStore *skstore, GtkTreeView *view,
     return column;
 }
 
-static void  
+/* The following three functions taken from bugzilla
+ * (http://bugzilla.gnome.org/attachment.cgi?id=49362&action=view)
+ * Author: Christian Neumair
+ * Copyright: 2005 Free Software Foundation, Inc
+ * License: GPL */
+static char *
+xds_get_atom_value (GdkDragContext *context)
+{
+	char *ret;
+
+	g_return_val_if_fail (context != NULL, NULL);
+	g_return_val_if_fail (context->source_window != NULL, NULL);
+
+	gdk_property_get (context->source_window,
+			  XDS_ATOM, TEXT_ATOM,
+			  0, MAX_XDS_ATOM_VAL_LEN,
+			  FALSE, NULL, NULL, NULL,
+			  (unsigned char **) &ret);
+
+	return ret;
+}
+
+static gboolean
+xds_context_offers_target (GdkDragContext *context, GdkAtom target)
+{
+	return (g_list_find (context->targets, target) != NULL);
+}
+
+static gboolean
+xds_is_dnd_valid_context (GdkDragContext *context)
+{
+	char *tmp;
+	gboolean ret;
+
+	g_return_val_if_fail (context != NULL, FALSE);
+
+	tmp = NULL;
+	if (xds_context_offers_target (context, XDS_ATOM)) {
+		tmp = xds_get_atom_value (context);
+	}
+
+	ret = (tmp != NULL);
+	g_free (tmp);
+
+	return ret;
+}
+
+static gboolean
 drag_begin (GtkWidget *widget, GdkDragContext *context, SeahorseKeyManagerStore *skstore)
 {
-    GtkTreeView *view = GTK_TREE_VIEW (widget);
-    SeahorseKeySource *sksrc;
-    SeahorseMultiOperation *mop = NULL;
-    SeahorseOperation *op = NULL;
-    GList *next, *keys, *sel_keys = NULL;
-    gpgme_data_t data = NULL;
-    SeahorseKey *skey;
+	GtkTreeView *view = GTK_TREE_VIEW (widget);
     
-    DBG_PRINT (("drag_begin -->\n"));
-    
-    sel_keys = seahorse_key_manager_store_get_selected_keys (view);
-    if(sel_keys != NULL) {
-        
-        /* Sort by key source */
-        keys = g_list_copy (sel_keys);
-        keys = seahorse_util_keylist_sort (keys);
-    
-        while (keys) {
-     
-            /* Break off one set (same keysource) */
-            next = seahorse_util_keylist_splice (keys);
+	DBG_PRINT (("drag_begin -->\n"));
 
-            g_assert (SEAHORSE_IS_KEY (keys->data));
-            skey = SEAHORSE_KEY (keys->data);
+	g_free (skstore->priv->drag_destination);
+	skstore->priv->drag_destination = NULL;
+	
+	g_clear_error (&skstore->priv->drag_error);
+	
+	g_list_free (skstore->priv->drag_objects);
+	skstore->priv->drag_objects = seahorse_key_manager_store_get_selected_objects (view);
+	if (skstore->priv->drag_objects)
+		gdk_property_change (context->source_window, XDS_ATOM, TEXT_ATOM, 
+		                     8, GDK_PROP_MODE_REPLACE, (guchar*)XDS_FILENAME,
+		                     strlen (XDS_FILENAME));
+	
+	DBG_PRINT (("drag_begin <--\n"));
+	return skstore->priv->drag_objects ? TRUE : FALSE;
+}
 
-            /* Export from this key source */        
-            sksrc = seahorse_key_get_source (skey);
-            g_return_if_fail (sksrc != NULL);
-            
-            if (!mop) 
-                mop = seahorse_multi_operation_new ();
-            
-            /* The data object where we export to */
-            if (!data) {
-                data = gpgmex_data_new ();
-                g_object_set_data_full (G_OBJECT (mop), "result-data", data,
-                                        (GDestroyNotify)gpgmex_data_release);
-            }
-        
-            /* We pass our own data object, to which data is appended */
-            op = seahorse_key_source_export (sksrc, keys, FALSE, data);
-            g_return_if_fail (op != NULL);
+static gboolean
+export_keys_to_output (GList *objects, GOutputStream *output, GError **error)
+{
+	SeahorseMultiOperation *mop = NULL;
+	SeahorseOperation *op;
+	SeahorseSource *sksrc;
+	SeahorseObject *sobj;
+	GList *next;
+	gboolean ret;
+	
+	objects = seahorse_util_objects_sort (objects);
+	DBG_PRINT (("exporting %d objects\n", g_list_length (objects)));
+	
+	while (objects) {
+		
+		/* Break off one set (same keysource) */
+		next = seahorse_util_objects_splice (objects);
 
-            g_list_free (keys);
-            keys = next;
+		g_assert (SEAHORSE_IS_OBJECT (objects->data));
+		sobj = SEAHORSE_OBJECT (objects->data);
 
-            seahorse_multi_operation_take (mop, op);
-        }
-        
-        g_object_set_data_full (G_OBJECT (view), "drag-operation", mop,
-                                (GDestroyNotify)g_object_unref);
-        g_object_set_data_full (G_OBJECT (view), "drag-keys", sel_keys,
-                                (GDestroyNotify)g_list_free);
-        g_object_set_data (G_OBJECT (view), "drag-file", NULL);
-    }
-    
-    DBG_PRINT (("drag_begin <--\n"));
+		/* Export from this key source */        
+		sksrc = seahorse_object_get_source (sobj);
+		g_return_val_if_fail (sksrc != NULL, FALSE);
+
+		if (!mop) 
+			mop = seahorse_multi_operation_new ();
+
+		/* We pass our own data object, to which data is appended */
+		op = seahorse_source_export (sksrc, objects, output);
+		g_return_val_if_fail (op != NULL, FALSE);
+
+		seahorse_multi_operation_take (mop, op);
+		
+		g_list_free (objects);
+		objects = next;		
+	}
+	
+	/* Make sure it's complete before we can return data */
+	op = SEAHORSE_OPERATION (mop);
+	seahorse_operation_wait (op);
+
+	ret = TRUE;
+	if (!seahorse_operation_is_successful (op)) { 
+		seahorse_operation_copy_error (op, error);
+		ret = FALSE;
+	}
+	
+	g_object_unref (mop);
+	return ret;
+}
+
+static gboolean
+export_to_text (SeahorseKeyManagerStore *skstore, GtkSelectionData *selection_data)
+{
+	GOutputStream *output;
+	gboolean ret;
+	GList *keys;
+	
+	ret = FALSE;
+	
+	g_return_val_if_fail (skstore->priv->drag_objects, FALSE);
+	keys = g_list_copy (skstore->priv->drag_objects);
+
+	DBG_PRINT (("exporting to text\n"));
+
+	output = g_memory_output_stream_new (NULL, 0, g_realloc, g_free);
+	g_return_val_if_fail (output, FALSE);
+
+	/* This modifies and frees keys */
+	ret = export_keys_to_output (keys, output, &skstore->priv->drag_error) &&
+	       g_output_stream_close (output, NULL, &skstore->priv->drag_error);
+	if (ret) {
+		DBG_PRINT (("setting selection text\n"));
+		gtk_selection_data_set_text (selection_data, 
+		                             g_memory_output_stream_get_data (G_MEMORY_OUTPUT_STREAM (output)), 
+		                             seahorse_util_memory_output_length (G_MEMORY_OUTPUT_STREAM (output)));
+	} else {
+		g_message ("error occurred on export: %s", 
+		           skstore->priv->drag_error && skstore->priv->drag_error->message ? 
+		                      skstore->priv->drag_error->message : "");
+	}
+	
+	g_object_unref (output);
+	return ret;
+}
+
+static gboolean
+export_to_filename (SeahorseKeyManagerStore *skstore, const gchar *filename)
+{
+	GOutputStream *output;
+	gboolean ret;
+	gchar *uri;
+	GFile *file;
+	GList *keys;
+	
+	DBG_PRINT (("exporting to %s\n", filename));
+
+	ret = FALSE;
+	g_return_val_if_fail (skstore->priv->drag_objects, FALSE);
+	keys = g_list_copy (skstore->priv->drag_objects);
+	
+	uri = seahorse_util_uri_unique (filename);
+
+	/* Create output file */
+	file = g_file_new_for_uri (uri);
+	g_free (uri);
+	output = G_OUTPUT_STREAM (g_file_replace (file, NULL, FALSE, G_FILE_CREATE_NONE, 
+	                                          NULL, &skstore->priv->drag_error));
+	g_object_unref (file);
+	
+	if (output) {
+		/* This modifies and frees keys */
+		ret = export_keys_to_output (keys, output, &skstore->priv->drag_error) &&
+		      g_output_stream_close (output, NULL, &skstore->priv->drag_error);
+		
+		g_object_unref (output);
+	}
+	
+	return ret;
+}
+
+static gboolean
+drag_data_get (GtkWidget *widget, GdkDragContext *context, GtkSelectionData *selection_data, 
+               guint info, guint time, SeahorseKeyManagerStore *skstore)
+{
+	gchar *destination;
+	gboolean ret;
+
+	DBG_PRINT (("drag_data_get %d -->\n", info)); 
+	
+	g_return_val_if_fail (skstore->priv->drag_objects, FALSE);
+
+	/* The caller wants plain text */
+	if (info == DRAG_INFO_TEXT) {
+		DBG_PRINT (("returning object text\n"));
+		export_to_text (skstore, selection_data);
+		
+	/* The caller wants XDS */
+	} else if (info == DRAG_INFO_XDS) {
+		
+		if (xds_is_dnd_valid_context (context)) {
+			destination = xds_get_atom_value (context);
+			g_return_val_if_fail (destination, FALSE);
+			skstore->priv->drag_destination = g_path_get_dirname (destination);
+			g_free (destination);
+		
+			gtk_selection_data_set (selection_data, selection_data->target, 8, (guchar*)"S", 1);
+			ret = TRUE;
+		}
+		
+	/* Unrecognized format */
+	} else {
+		DBG_PRINT (("Unrecognized format: %d\n", info));
+	}
+	
+	DBG_PRINT(("drag_data_get <--\n"));
+	return ret;
 }
 
 static void
-cleanup_file (gchar *file)
-{
-    g_assert (file != NULL);
-    DBG_PRINT (("deleting temp file: %s\n", file));
-    unlink (file);
-    g_free (file);
-}
-
-static void  
 drag_end (GtkWidget *widget, GdkDragContext *context, SeahorseKeyManagerStore *skstore)
 {
-    DBG_PRINT (("drag_end -->\n"));
-    
-    /* This frees the operation and key list if present */
-    g_object_set_data (G_OBJECT (widget), "drag-operation", NULL);
-    g_object_set_data (G_OBJECT (widget), "drag-keys", NULL);
-
-    DBG_PRINT (("drag_end <--\n"));
-}
-
-static void  
-drag_data_get (GtkWidget *widget, GdkDragContext *context, 
-               GtkSelectionData *selection_data, guint info, 
-               guint time, SeahorseKeyManagerStore *skstore)
-{
-    SeahorseOperation *op;
-    gchar *t, *n;
-    GList *keys;
-    GError *err = NULL;
-    gpgme_data_t data;
-    gchar *text;
-
-    DBG_PRINT (("drag_data_get %d -->\n", info)); 
-
-    op = (SeahorseOperation*)g_object_get_data (G_OBJECT (widget), "drag-operation");
-    if (op == NULL) {
-        DBG_PRINT (("No operation in drag"));
-        return;
-    }
-
-    /* Make sure it's complete before we can return data */
-    seahorse_operation_wait (op);
-
-    if (!seahorse_operation_is_successful (op)) {
-        g_object_set_data (G_OBJECT (widget), "drag-operation", NULL);
-        seahorse_operation_copy_error (op, &err);
-        seahorse_util_handle_error (err, _("Couldn't retrieve key data"));
-        return;
-    }
-    
-    data = (gpgme_data_t)g_object_get_data (G_OBJECT (op), "result-data");
-    g_return_if_fail (data != NULL);
-    
-    text = seahorse_util_write_data_to_text (data, NULL);
-    g_return_if_fail (text != NULL);
-
-    if (info == TEXT_PLAIN) {
-        DBG_PRINT (("returning key text\n"));
-        gtk_selection_data_set_text (selection_data, text, strlen (text));
-    } else if (info == TEXT_URIS) {
-        t = (gchar*)g_object_get_data (G_OBJECT (widget), "drag-file");
-
-        if (t == NULL) {
-            keys = g_object_get_data (G_OBJECT (widget), "drag-keys");
-            g_return_if_fail (keys != NULL);
-            
-            n = seahorse_util_filename_for_keys (keys);
-            g_return_if_fail (n != NULL);
-            t = g_build_filename(g_get_tmp_dir (), n, NULL);
-            g_free (n);
-
-            g_object_set_data_full (G_OBJECT (widget), "drag-file", t,
-                                    (GDestroyNotify)cleanup_file);
-            
-            DBG_PRINT (("writing to temp file: %s\n", t));
-            
-            if (!seahorse_vfs_set_file_contents (t, text, strlen (text), &err)) {
-                seahorse_util_handle_error (err, _("Couldn't write key to file"));
-                g_object_set_data (G_OBJECT (widget), "drag-file", NULL);
-                t = NULL;
-            }
-        }
-
-        if (t != NULL) {
-            char *uris[2] = { t, NULL };
-            gtk_selection_data_set_uris (selection_data, uris);
-        }
-    }
-    
-    DBG_PRINT(("drag_data_get <--\n"));
-
-    g_free(text);
+	gchar *filename, *name;
+	
+	DBG_PRINT (("drag_end -->\n"));
+	
+	if (skstore->priv->drag_destination && !skstore->priv->drag_error) {
+		g_return_if_fail (skstore->priv->drag_objects);
+	
+		name = seahorse_util_filename_for_keys (skstore->priv->drag_objects);
+		g_return_if_fail (name);
+	
+		filename = g_build_filename (skstore->priv->drag_destination, name, NULL);
+		g_free (name);
+	
+		export_to_filename (skstore, filename);
+		g_free (filename);
+	}
+	
+	if (skstore->priv->drag_error) {
+		seahorse_util_show_error (widget, _("Couldn't export keys"),
+		                          skstore->priv->drag_error->message);
+	}
+	
+	g_clear_error (&skstore->priv->drag_error);
+	g_list_free (skstore->priv->drag_objects);
+	skstore->priv->drag_objects = NULL;
+	g_free (skstore->priv->drag_destination);
+	skstore->priv->drag_destination = NULL;
+	
+	DBG_PRINT (("drag_end <--\n"));
 }
 
 static gint 
@@ -749,14 +671,6 @@ seahorse_key_manager_store_init (SeahorseKeyManagerStore *skstore)
 {
     /* init private vars */
     skstore->priv = g_new0 (SeahorseKeyManagerStorePriv, 1);
-    
-    /* Setup the store */
-    seahorse_key_model_set_column_types (SEAHORSE_KEY_MODEL (skstore), N_COLS, (GType *) col_types);
-    
-    /* Setup the sort and filter */
-    skstore->priv->filter = GTK_TREE_MODEL_FILTER (gtk_tree_model_filter_new (GTK_TREE_MODEL (skstore), NULL));
-    gtk_tree_model_filter_set_visible_func (skstore->priv->filter, filter_callback, skstore, NULL);
-    skstore->priv->sort = GTK_TREE_MODEL_SORT (gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (skstore->priv->filter)));
 }
 
 static void
@@ -766,10 +680,7 @@ seahorse_key_manager_store_get_property (GObject *gobject, guint prop_id,
     SeahorseKeyManagerStore *skstore = SEAHORSE_KEY_MANAGER_STORE (gobject);
     
     switch (prop_id) {
-    case PROP_KEYSET:
-        g_value_set_object (value, skstore->skset);
-        break;
-        
+
     /* The filtering mode */
     case PROP_MODE:
         g_value_set_uint (value, skstore->priv->filter_mode);
@@ -795,14 +706,6 @@ seahorse_key_manager_store_set_property (GObject *gobject, guint prop_id,
     const gchar* t;
     
     switch (prop_id) {
-    case PROP_KEYSET:
-        g_assert (skstore->skset == NULL);
-        skstore->skset = g_value_get_object (value);
-        g_object_ref (skstore->skset);
-        g_signal_connect_after (skstore->skset, "added", G_CALLBACK (key_added), skstore);
-        g_signal_connect_after (skstore->skset, "removed", G_CALLBACK (key_removed), skstore);
-        g_signal_connect_after (skstore->skset, "changed", G_CALLBACK (key_changed), skstore);
-        break;
 
     /* The filtering mode */
     case PROP_MODE:
@@ -837,37 +740,10 @@ seahorse_key_manager_store_set_property (GObject *gobject, guint prop_id,
     }
 }
 
-static void
-seahorse_key_manager_store_dispose (GObject *gobject)
-{
-    SeahorseKeyManagerStore *skstore;
-    
-    /*
-     * Note that after this executes the rest of the object should
-     * still work without a segfault. This basically nullifies the 
-     * object, but doesn't free it.
-     * 
-     * This function should also be able to run multiple times.
-     */
-
-    skstore = SEAHORSE_KEY_MANAGER_STORE (gobject);  
-
-    if (skstore->skset) {
-        g_signal_handlers_disconnect_by_func (skstore->skset, key_added, skstore);
-        g_signal_handlers_disconnect_by_func (skstore->skset, key_removed, skstore);
-        g_signal_handlers_disconnect_by_func (skstore->skset, key_changed, skstore);
-        g_object_unref (skstore->skset);        
-        skstore->skset = NULL;
-    }
-    
-    G_OBJECT_CLASS (seahorse_key_manager_store_parent_class)->dispose (gobject);
-}
-    
 static void
 seahorse_key_manager_store_finalize (GObject *gobject)
 {
     SeahorseKeyManagerStore *skstore = SEAHORSE_KEY_MANAGER_STORE (gobject);
-    g_assert (skstore->skset == NULL);
     
     /* These were allocated in the constructor */
     g_object_unref (skstore->priv->sort);
@@ -886,18 +762,12 @@ seahorse_key_manager_store_class_init (SeahorseKeyManagerStoreClass *klass)
     seahorse_key_manager_store_parent_class = g_type_class_peek_parent (klass);
     
     /* Some simple checks to make sure all column info is on the same page */
-    g_assert (N_COLS == G_N_ELEMENTS (col_ids));
-    g_assert (N_COLS == G_N_ELEMENTS (col_types));
+    g_assert (N_COLS == G_N_ELEMENTS (column_info));
 
     gobject_class->finalize = seahorse_key_manager_store_finalize;
-    gobject_class->dispose = seahorse_key_manager_store_dispose;
     gobject_class->set_property = seahorse_key_manager_store_set_property;
     gobject_class->get_property = seahorse_key_manager_store_get_property;
   
-    g_object_class_install_property (gobject_class, PROP_KEYSET,
-        g_param_spec_object ("keyset", "Seahorse Keyset", "Current Seahorse Key Source to use",
-                             SEAHORSE_TYPE_KEYSET, G_PARAM_READWRITE));
-
     g_object_class_install_property (gobject_class, PROP_MODE,
         g_param_spec_uint ("mode", "Key Store Mode", "Key store mode controls which keys to display",
                            0, KEY_STORE_MODE_FILTERED, KEY_STORE_MODE_ALL, G_PARAM_READWRITE));
@@ -912,21 +782,25 @@ seahorse_key_manager_store_class_init (SeahorseKeyManagerStoreClass *klass)
  */
 
 SeahorseKeyManagerStore*
-seahorse_key_manager_store_new (SeahorseKeyset *skset, GtkTreeView *view)
+seahorse_key_manager_store_new (SeahorseSet *skset, GtkTreeView *view)
 {
     SeahorseKeyManagerStore *skstore;
     GtkTreeViewColumn *col;
-    SeahorseKeyPredicate *pred;
+    SeahorseObjectPredicate *pred;
     GtkCellRenderer *renderer;
     gchar *sort;
-    GtkTargetList *targets;
+    gint last;
     
-    skstore = g_object_new (SEAHORSE_TYPE_KEY_MANAGER_STORE, "keyset", skset, NULL);
+    skstore = g_object_new (SEAHORSE_TYPE_KEY_MANAGER_STORE, "set", skset, NULL);
+    last = seahorse_set_model_set_columns (SEAHORSE_SET_MODEL (skstore), column_info, N_COLS);
+    g_return_val_if_fail (last == N_COLS - 1, NULL);
 
-    populate_store (skstore);
+    /* Setup the sort and filter */
+    skstore->priv->filter = GTK_TREE_MODEL_FILTER (gtk_tree_model_filter_new (GTK_TREE_MODEL (skstore), NULL));
+    gtk_tree_model_filter_set_visible_func (skstore->priv->filter, filter_callback, skstore, NULL);
+    skstore->priv->sort = GTK_TREE_MODEL_SORT (gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (skstore->priv->filter)));
 
     /* The sorted model is the top level model */
-    g_assert (GTK_IS_TREE_MODEL (skstore->priv->sort));
     gtk_tree_view_set_model (view, GTK_TREE_MODEL (skstore->priv->sort));
 
     /* add the icon column */
@@ -936,7 +810,7 @@ seahorse_key_manager_store_new (SeahorseKeyset *skset, GtkTreeView *view)
     col = gtk_tree_view_column_new_with_attributes ("", renderer, "stock-id", COL_STOCK_ID, NULL);
     gtk_tree_view_column_set_resizable (col, FALSE);
     gtk_tree_view_append_column (view, col);
-    gtk_tree_view_column_set_sort_column_id (col, COL_PAIR);
+    gtk_tree_view_column_set_sort_column_id (col, COL_USAGE);
     
     /* Name column */
     renderer = gtk_cell_renderer_text_new ();
@@ -951,13 +825,13 @@ seahorse_key_manager_store_new (SeahorseKeyset *skset, GtkTreeView *view)
     g_object_get (skset, "predicate", &pred, NULL);
     
     /* Key ID column, don't show for passwords */
-    if (pred->etype != SKEY_CREDENTIALS) {
+    if (pred->usage != SEAHORSE_USAGE_CREDENTIALS) {
         col = append_text_column (skstore, view, _("Key ID"), COL_KEYID);
         gtk_tree_view_column_set_sort_column_id (col, COL_KEYID);
     }
 
     /* Public keys show validity */
-    if (pred->etype == SKEY_PUBLIC) {
+    if (pred->usage == SEAHORSE_USAGE_PUBLIC_KEY) {
         col = append_text_column (skstore, view, _("Validity"), COL_VALIDITY_STR);
         g_object_set_data (G_OBJECT (col), "gconf-key", SHOW_VALIDITY_KEY);
         gtk_tree_view_column_set_visible (col, seahorse_gconf_get_boolean (SHOW_VALIDITY_KEY));
@@ -1001,19 +875,14 @@ seahorse_key_manager_store_new (SeahorseKeyset *skset, GtkTreeView *view)
     g_signal_connect (G_OBJECT (view), "drag_begin",  G_CALLBACK (drag_begin), skstore);
     g_signal_connect (G_OBJECT (view), "drag_end",  G_CALLBACK (drag_end), skstore);
 
-    gtk_drag_source_set (GTK_WIDGET (view), GDK_BUTTON1_MASK | GDK_BUTTON2_MASK,
-                         NULL, 0, GDK_ACTION_COPY);
-    targets = gtk_target_list_new (NULL, 0);
-    gtk_target_list_add_uri_targets (targets, TEXT_URIS);
-    gtk_target_list_add_text_targets (targets, TEXT_PLAIN);
-    gtk_drag_source_set_target_list (GTK_WIDGET (view), targets);
-    gtk_target_list_unref (targets);
+    gtk_drag_source_set (GTK_WIDGET (view), GDK_BUTTON1_MASK,
+                         store_targets, G_N_ELEMENTS (store_targets), GDK_ACTION_COPY);
 
     return skstore;
 }
 
-SeahorseKey*
-seahorse_key_manager_store_get_key_from_path (GtkTreeView *view, GtkTreePath *path, guint *uid)
+SeahorseObject*
+seahorse_key_manager_store_get_object_from_path (GtkTreeView *view, GtkTreePath *path)
 {
     GtkTreeModel *model;
     GtkTreeIter iter;
@@ -1023,24 +892,24 @@ seahorse_key_manager_store_get_key_from_path (GtkTreeView *view, GtkTreePath *pa
     
     model = gtk_tree_view_get_model (view);
     g_return_val_if_fail (gtk_tree_model_get_iter (model, &iter, path), NULL);
-    return key_from_iterator (model, &iter, uid);
+    return object_from_iterator (model, &iter);
 }
 
 GList*
-seahorse_key_manager_store_get_all_keys (GtkTreeView *view)
+seahorse_key_manager_store_get_all_objects (GtkTreeView *view)
 {
     SeahorseKeyManagerStore* skstore;
     
     g_return_val_if_fail (GTK_IS_TREE_VIEW (view), NULL);
     skstore = key_store_from_model (gtk_tree_view_get_model (view));
-    return seahorse_keyset_get_keys (skstore->skset);
+    return seahorse_set_get_objects (SEAHORSE_SET_MODEL (skstore)->set);
 }
 
 GList*
-seahorse_key_manager_store_get_selected_keys (GtkTreeView *view)
+seahorse_key_manager_store_get_selected_objects (GtkTreeView *view)
 {
-    SeahorseKey *skey;
-    GList *l, *keys = NULL;
+    SeahorseObject *sobj;
+    GList *l, *objects = NULL;
     SeahorseKeyManagerStore* skstore;
     GList *list, *paths = NULL;
     GtkTreeSelection *selection;    
@@ -1052,11 +921,11 @@ seahorse_key_manager_store_get_selected_keys (GtkTreeView *view)
     selection = gtk_tree_view_get_selection (view);
     paths = gtk_tree_selection_get_selected_rows (selection, NULL);
 
-    /* make key list */
+    /* make object list */
     for (list = paths; list != NULL; list = g_list_next (list)) {
-        skey = seahorse_key_manager_store_get_key_from_path (view, list->data, NULL);
-        if (skey != NULL)
-            keys = g_list_append (keys, skey);
+        sobj = seahorse_key_manager_store_get_object_from_path (view, list->data);
+        if (sobj != NULL)
+            objects = g_list_append (objects, sobj);
     }
         
     /* free selected paths */
@@ -1064,81 +933,75 @@ seahorse_key_manager_store_get_selected_keys (GtkTreeView *view)
     g_list_free (paths);
     
     /* Remove duplicates */
-    keys = g_list_sort (keys, compare_pointers);
-    for (l = keys; l; l = g_list_next (l)) {
+    objects = g_list_sort (objects, compare_pointers);
+    for (l = objects; l; l = g_list_next (l)) {
         while (l->next && l->data == l->next->data)
-            keys = g_list_delete_link (keys, l->next);
+            objects = g_list_delete_link (objects, l->next);
     }    
     
-    return keys;
+    return objects;
 }
 
 void
-seahorse_key_manager_store_set_selected_keys (GtkTreeView *view, GList* keys)
+seahorse_key_manager_store_set_selected_objects (GtkTreeView *view, GList* objects)
 {
-    SeahorseKeyManagerStore* skstore;
-    GtkTreeSelection* selection;
-    gboolean first = TRUE;
-    GtkTreePath *path;
-    GList *l;
-    GSList *rows, *rl;
-    GtkTreeIter it;
+	SeahorseKeyManagerStore* skstore;
+	GtkTreeSelection* selection;
+	gboolean first = TRUE;
+	GtkTreePath *path;
+	GList *l;
+	GtkTreeIter iter, upper;
     
-    g_return_if_fail (GTK_IS_TREE_VIEW (view));
-    selection = gtk_tree_view_get_selection (view);
-    gtk_tree_selection_unselect_all (selection);
+	g_return_if_fail (GTK_IS_TREE_VIEW (view));
+	selection = gtk_tree_view_get_selection (view);
+	gtk_tree_selection_unselect_all (selection);
     
-    skstore = key_store_from_model (gtk_tree_view_get_model (view));
-    g_return_if_fail (SEAHORSE_IS_KEY_MANAGER_STORE (skstore));
+	skstore = key_store_from_model (gtk_tree_view_get_model (view));
+	g_return_if_fail (SEAHORSE_IS_KEY_MANAGER_STORE (skstore));
     
-    for(l = keys; l; l = g_list_next (l)) {
-        
-        /* Get all the rows for that key .... */
-        rows = seahorse_key_model_get_rows_for_key (SEAHORSE_KEY_MODEL (skstore), 
-                                                    SEAHORSE_KEY (l->data));
-        for(rl = rows; rl; rl = g_slist_next (rl)) {
+	for (l = objects; l; l = g_list_next (l)) {
+		if (seahorse_set_model_iter_for_object (SEAHORSE_SET_MODEL (skstore), 
+		                                        SEAHORSE_OBJECT (l->data), &iter)) {
+			/* And select them ... */
+			if (get_upper_iter (skstore, &upper, &iter)) {
 
-            /* And select them ... */
-            if (get_upper_iter(skstore, &it, (GtkTreeIter*)rl->data)) {
-                gtk_tree_selection_select_iter (selection, &it);
-                
-                /* Scroll the first row selected into view */
-                if (first) {
-                    path = gtk_tree_model_get_path (gtk_tree_view_get_model (view), &it);
-                    gtk_tree_view_scroll_to_cell (view, path, NULL, FALSE, 0.0, 0.0);
-                    gtk_tree_path_free (path);
-                    first = FALSE;
-                }
-            }
-        }
-        
-        seahorse_key_model_free_rows (rows);
-    }
+				gtk_tree_selection_select_iter (selection, &upper);
+		                
+				/* Scroll the first row selected into view */
+				if (first) {
+					path = gtk_tree_model_get_path (gtk_tree_view_get_model (view), &upper);
+					gtk_tree_view_scroll_to_cell (view, path, NULL, FALSE, 0.0, 0.0);
+					gtk_tree_path_free (path);
+					first = FALSE;
+				}
+			}
+		}
+	}
 }
 
-SeahorseKey*
-seahorse_key_manager_store_get_selected_key (GtkTreeView *view, guint *uid)
+SeahorseObject*
+seahorse_key_manager_store_get_selected_object (GtkTreeView *view)
 {
-    SeahorseKeyManagerStore* skstore;
-    SeahorseKey *skey = NULL;
-    GList *paths = NULL;
-    GtkTreeSelection *selection;
+	SeahorseKeyManagerStore* skstore;
+	SeahorseObject *sobj = NULL;
+	GList *paths = NULL;
+	GtkTreeSelection *selection;
     
-    g_return_val_if_fail (GTK_IS_TREE_VIEW (view), NULL);
-    skstore = key_store_from_model (gtk_tree_view_get_model (view));
-    g_return_val_if_fail (SEAHORSE_IS_KEY_MANAGER_STORE (skstore), NULL);
+	g_return_val_if_fail (GTK_IS_TREE_VIEW (view), NULL);
+	skstore = key_store_from_model (gtk_tree_view_get_model (view));
+	g_return_val_if_fail (SEAHORSE_IS_KEY_MANAGER_STORE (skstore), NULL);
     
-    selection = gtk_tree_view_get_selection (view);
-    paths = gtk_tree_selection_get_selected_rows (selection, NULL);
+	selection = gtk_tree_view_get_selection (view);
+	paths = gtk_tree_selection_get_selected_rows (selection, NULL);
     
-    /* make key list */
-    if (paths != NULL) {
-        skey = seahorse_key_manager_store_get_key_from_path (view, paths->data, uid);
+	/* choose first object */
+	if (paths != NULL) {
+		sobj = seahorse_key_manager_store_get_object_from_path (view, paths->data);
             
-        /* free selected paths */
-        g_list_foreach (paths, (GFunc)gtk_tree_path_free, NULL);
-        g_list_free (paths);
-    }
+		/* free selected paths */
+		g_list_foreach (paths, (GFunc)gtk_tree_path_free, NULL);
+		g_list_free (paths);
+	}
     
-    return skey;
+	return sobj;
 }
